/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*! \file MainWindow.h
    \brief Implements interface for GUI

    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "../../include/GUIInterface.h"
#include "../../include/settings.h"
#include "../backbone/backbone.h"
#include "TranslationWidget.h"
#include "WordListWidget.h"
#include "SearchBarWidget.h"
#include "MenuWidget.h"
#include "DictManagerWidget.h"
#include "SettingsWidget.h"
#include "HistoryListDialog.h"
#include "BookmarksWidget.h"
#include "WelcomeScreenWidget.h"
#include "AboutWidget.h"
#include "NotifyManager.h"


/*!
  Creates all of GUI subcomponents, and connects all GUI interface signals
  with suitable backbone signals and slots.
  Only this class has direct access to backbone object.
  It manages all requests of subcomponents e. g. searching for a given word,
  displaying history, removing a dictionary.
  It also provides data from backbone to subcomponents e. g. result of search.
*/
class MainWindow : public GUIInterface
{
    Q_OBJECT

public:
    //! Constructor
    /*!
      \param backbone backbone object which will be doing all searches and returning data
      \param parent parent widget of this window
    */
    MainWindow(Backbone* backbone, QWidget *parent = 0);
    ~MainWindow();

    //! Returns all loaded dictionaries with information if they are
    //! active/inactive
    /*!
        \return Hash of pointers to a dictionary and boolean flag indicating if
        a dictionary is active
    */
    QHash<CommonDictInterface*, bool> getDictionaries();

    //! Returns all loaded plugins
    /*!
        \return List of pointers to plugins
    */
    QList<CommonDictInterface*> getPlugins();

    //! Indicates if GUI is in exact search mode.
    /*! When GUI is in exact search mode it searches for a word, and if it
        finds exactly matching translation it displays matching words list
        and then directly displays translation of first exactly matched word.
        This mode is used for browsing search history and searching words
        from application arguments.
        \returns flag indicating if GUI is in exact search mode
        \sa setExactSearch()
        \sa setExactSearchString()
    */
    bool isInExactSearch();


    //! Returns current application settings.
    /*!
       \returns Settings object containing current application settings
    */
    Settings* settings();


    //! Sets new settings.
    /*!
       \param Settings object containing new application settings
    */
    void setSettings(Settings*);


 public Q_SLOTS:
    //! Searches in exact mode for a given word
    /*!
      GUI will be automatically set into exact search mode, and after search or
      when user breaks the search it will be unset from exact search mode.
      \param word which will be searched for in dictionaries
      \sa search()
    */
    void searchExact(QString);


    //! Searches for a given word
    /*!
      It sets passed word in line edit of a search bar and searches for a given word.
      \param word which will be searched for in dictionaries
      \sa SearchBarWidget
      \sa searchExact()
    */
    void search(QString);

    //! Starts searching for a given word after 500 ms delay
    /*!
      After time's up it sets passed word in line edit of search bar and searches
      for a given word.
      \param word which will be searched for in dictionaries
      \sa SearchBarWidget
      \sa searchExact()
      \sa search()
    */
    void searchDelay(QString);

    //! Sets string for exact search
    /*!
        Sets string for which current search is ongoing, is used to find exact
        word when GUI is in search exact mode.
    */
    void setExactSearchString(QString);

    //! Sets GUI exact search mode.
    /*! When GUI is in exact search mode it searches for word, and if it
        finds exactly matching translation it displays matching words list
        and then directly displays translation of first exactly matched word.
        This mode is used for browsing search history and searching words
        from application arguments.
        \param exactSearch flag indicating if GUI is in exact search mode
        \sa isInExactSearch()
        \sa setExactSearchString()
    */
    void setExactSearch(bool exactSearch);


    //! Gets word list from backbone and prepares received list to display
    /*!
      Checks if received list is empty, in that case it displays suitable
      information. Otherwise it merges results of the same key word and emits
      signal to display word list.
      If GUI is in exact search mode it will search for an exact word in received
      list and if any of found words match exactly the word passed to
      searchExact() method.
      \sa isInExactSearch()
      \sa searchExact()
      \sa showTranslation()
      \sa setExactSearchString()
     */
    void wordListReady();

    //! Gets translation strings from backbone and emits signal to display them
    void translationsReady();

    //! Adds key words from given translations to history
    /*!
      By default this slot is connected to searchTranslations signal, and
      passed translation list contains only translations with the same key, so
      only one word is added to history.
      \param list of translations with key words
      \sa searchTranslations()
      */
    void addToHistory(QList<Translation*>);

    //! Shows history dialog
    /*!
       In maemo it shows dialog with history.
       In desktop it shows popup containing history, whose bottom edge is on the
       same height as passed point.
       \param point on screen where popup has to show
    */
    void showHistory(QPoint);

    //! Shows translation of next word in history
    /*!
      It will work only if there is next word available in history.
      Translation of a word is searched for with searchDelay() function.
      On maemo search is in normal mode, on desktop in exact search mode.
      \sa searchDelay()
      \sa searchExact()
      */
    void historyNext();

    //! Shows translation of previous word in history
    /*!
      It will work only if there is previous word available in history.
      Translation of a word is searched for with searchDelay() function.
      On maemo search is in normal mode, on desktop in exact search mode.
      \sa searchDelay()
      \sa searchExact()
      */
    void historyPrev();


    //! Shows notification to user
    /*!
      It shows different types of notifications such as information, warnings and errors.
      In maemo they are represented as notes, on desktop as message boxes.
      \param type type of notification
      \param message notification message
    */
    void showNotification(Notify::NotifyType type, QString message);

private Q_SLOTS:
    //! Disables menu
    void disableMenu();

    //! Enables menu
    void enableMenu();

    //! When user breaks searching it makes sure that exact search mode will be
    //! disabled
    void searchingInterrupted();

    //! Asks for confirmation when user clicks on "delete all bookmarks"
    void removeBookmarks();

protected:
    /*!
        When user wants to close application, we first send signal to stop all
        ongoing searches.
    */
    void closeEvent(QCloseEvent *);


private:
    Backbone* backbone;

    void initializeUI();
    void initializeSearchWidgets();
    void initializeMenu();
    void initializeMenuWidgets();

    void hideWelcomeScreen();

    bool checkExactSearch(QHash<QString, QList<Translation*> > searchResult,
                          QList<Translation*> &found);

    SearchBarWidget* searchBarWidget;
    QWidget* translationWidget;
    QWidget* wordListWidget;
    MenuWidget* menuWidget;
    DictManagerWidget* dictManagerWidget;
    SettingsWidget* settingsWidget;
    BookmarksWidget* bookmarksWidget;
    QWidget* welcomeScreenWidget;
    AboutWidget* aboutWidget;
    QMenuBar* menuBar;
    QVBoxLayout* mainLayout;
    NotifyManager* notifyManager;

    #ifndef Q_WS_MAEMO_5
        QSplitter* splitter;
        QAction* dictionariesAction;
        QAction* bookmarksShowAllAction;
        QAction* bookmarksRemoveAllAction;
        QAction* settingsAction;
        QAction* aboutAction;
    #endif

    bool _exactSearch;
    QString searchString;


    void connectBackbone();
    void connectSearchBar();
    void connectWordList();
    void connectTranslationWidget();
    void connectDictManager();
    void connectMenu();
    void connectBookmarksWidget();
};

#endif // MAINWINDOW_H
