/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*! \file SearchBarWidget.h
    \brief Displays search bar

    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/


#ifndef SEARCHBARWIDGET_H
#define SEARCHBARWIDGET_H

#include <QWidget>
#include <QtGui>
#include <QDeclarativeView>

#include "../backbone/backbone.h"
#include "../../include/History.h"

/*!
    Contains line edit field to input word which user would like to find
    and buttons to start/stop search and browse search history.
    Line edit and history buttons are disabled when search is ongoing,
    only start/stop button stays active. When searching it also displays
    progress bar.
*/
class SearchBarWidget : public QWidget {
    Q_OBJECT
public:
    explicit SearchBarWidget(QWidget *parent = 0);
    ~SearchBarWidget();

Q_SIGNALS:

    void setEnableHistoryNext(QVariant enable);
    void setEnableHistoryShow(QVariant enable);
    void setEnableHistoryPrev(QVariant enable);
    void setButtonText(QVariant text);
    void setLineEditText(QVariant text);
    void setLineEditEnables(QVariant enabled);

    //! Requests to search for a list of words matching a word passed as
    //! a parameter
    void searchForTranslations(QString);

    //! Requests to stop all active searchings
    void stopSearching();

    //! Requests to show previous translation in history
    void historyPrev();

    //! Requests to show next translation in history
    void historyNext();

    //! Requests to show history list
    /*!
      \param p this argument is used only on desktop, it defines place on
      which a popup with history will be shown
    */
    void historyShow(QPoint p = QPoint(-1,-1));

    //! Requests to refresh state of history buttons
    void refreshHistoryButtons();

public Q_SLOTS:

    void searchButtonClicked(QString text);

    //! Enables or disables search word line edit and history buttons
    /*!
      While searching it disables only history button and line edit.
      Search/Stop button is always enabled.
    */
    void setEnabled(bool);


    //! Sets search bar in busy state
    /*!
      Displays "busy" bar and disables search word text edit and history buttons
    */
    void setBusy();

    //! Sets search bar in idle state
    /*!
      Hides "busy" bar and enables all widgets, refreshes state of history buttons
      by emitting refreshHistoryButtons signal
    */
    void setIdle();

    //! Searches for a given word
    /*!
      Sets word as text in search word line edit
    */
    void search(QString word);

    //! Starts to search for a given word after 500 ms delay
    /*!
      Sets word as text in search word line edit, and waits 500 ms to start
      search. If in the meantime this slot is called again it will stop previous
      timers.
    */
    void searchDelay(QString word);

    //! Updates state of history buttons
    /*!
      \param prev if set to true, the history has some previous words
      \param next if set to true, the history has some next words
      \param list if set to true, the history can show word list
    */
    void updateHistoryButtons(bool prev, bool next, bool list);

    void setFocus();

private Q_SLOTS:
    //! Clears search word line edit
    void clearSearchWordToolButtonClicked();

    //! Starts to search for given words
    void searchPushButtonClicked();

    //! starts to search word which was passed to searchDelay
    void delaySearchTimeout();

    //! shows history
    void showHistoryButtonClicked();


private:

    QVBoxLayout* mainLayout;
    QDeclarativeView *view;

    QLineEdit* searchWordLineEdit;
    QCompleter* lineEditCompleter;
    QStringListModel* completerModel;
    QToolButton* clearSearchWordToolButton;
    QPushButton* searchPushButton;
    QToolButton* historyPrevToolButton;
    QToolButton* historyNextToolButton;
    QToolButton* historyShowToolButton;
    QToolButton* fullScreenToolButton;
    QHBoxLayout* horizontalLayout;
    QProgressBar* searchingProgressBar;


    //! generates icon for maemo (some of icons we use don't have inactive
    //! pixmaps, so we generate them)
    /*!
      \param original original icon
      \param rotation rotation of resulting icon
    */
    QIcon generateIcon(QIcon original, qreal rotation=0);

    QVBoxLayout* verticalLayout;

    bool busy;

    QTimer delayTimer;
    QString delayString;

    void initializeUI();
};

#endif // SEARCHBARWIDGET_H
